@@ -21,7 +21,7 @@ class Foreman::CLI < Thor
 
   method_option :color,     :type => :boolean, :aliases => "-c", :desc => "Force color to be enabled"
   method_option :env,       :type => :string,  :aliases => "-e", :desc => "Specify an environment file to load, defaults to .env"
-  method_option :formation, :type => :string,  :aliases => "-m", :banner => '"alpha=5,bar=3"'
+  method_option :formation, :type => :string,  :aliases => "-m", :banner => '"alpha=5,bar=3"', :desc => 'Specify what processes will run and how many. Default: "all=1"'
   method_option :port,      :type => :numeric, :aliases => "-p"
   method_option :timeout,   :type => :numeric, :aliases => "-t", :desc => "Specify the amount of time (in seconds) processes have to shutdown gracefully before receiving a SIGKILL, defaults to 5."
 
@@ -50,7 +50,8 @@ class Foreman::CLI < Thor
   method_option :port,        :type => :numeric, :aliases => "-p"
   method_option :user,        :type => :string,  :aliases => "-u"
   method_option :template,    :type => :string,  :aliases => "-t"
-  method_option :concurrency, :type => :string,  :aliases => "-c", :banner => '"alpha=5,bar=3"'
+  method_option :formation, :type => :string,  :aliases => "-m", :banner => '"alpha=5,bar=3"', :desc => 'Specify what processes will run and how many. Default: "all=1"'
+  method_option :timeout,     :type => :numeric, :aliases => "-t", :desc => "Specify the amount of time (in seconds) processes have to shutdown gracefully before receiving a SIGKILL, defaults to 5."
 
   def export(format, location=nil)
     check_procfile!
@@ -79,7 +80,7 @@ class Foreman::CLI < Thor
   def run(*args)
     load_environment!
 
-    if File.exist?(procfile)
+    if File.file?(procfile)
       engine.load_procfile(procfile)
     end
 
@@ -101,7 +102,7 @@ class Foreman::CLI < Thor
       Process.kill(:INT, pid)
     end
     Process.wait(pid)
-    exit $?.exitstatus
+    exit $?.exitstatus || 0
   rescue Interrupt
   end
 
@@ -129,7 +130,7 @@ private ######################################################################
   end
 
   def check_procfile!
-    error("#{procfile} does not exist.") unless File.exist?(procfile)
+    error("#{procfile} does not exist.") unless File.file?(procfile)
   end
 
   def load_environment!
@@ -139,7 +140,7 @@ private ######################################################################
       end
     else
       default_env = File.join(engine.root, ".env")
-      engine.load_env default_env if File.exists?(default_env)
+      engine.load_env default_env if File.file?(default_env)
     end
   end
 
@@ -153,9 +154,8 @@ private ######################################################################
 
   def options
     original_options = super
-    return original_options unless File.exists?(".foreman")
+    return original_options unless File.file?(".foreman")
     defaults = ::YAML::load_file(".foreman") || {}
     Thor::CoreExt::HashWithIndifferentAccess.new(defaults.merge(original_options))
   end
-
 end
