@@ -3,7 +3,6 @@ require "foreman/env"
 require "foreman/process"
 require "foreman/procfile"
 require "tempfile"
-require "timeout"
 require "fileutils"
 require "thread"
 
@@ -28,7 +27,7 @@ class Foreman::Engine
   def initialize(options={})
     @options = options.dup
 
-    @options[:formation] ||= (options[:concurrency] || "all=1")
+    @options[:formation] ||= "all=1"
     @options[:timeout] ||= 5
 
     @env       = {}
@@ -37,6 +36,7 @@ class Foreman::Engine
     @processes = []
     @running   = {}
     @readers   = {}
+    @shutdown  = false
 
     # Self-pipe for deferred signal-handling (ala djb: http://cr.yp.to/docs/selfpipe.html)
     reader, writer       = create_pipe
@@ -57,8 +57,9 @@ class Foreman::Engine
     spawn_processes
     watch_for_output
     sleep 0.1
-    watch_for_termination { terminate_gracefully }
+    wait_for_shutdown_or_child_termination
     shutdown
+    exit(@exitstatus) if @exitstatus
   end
 
   # Set up deferred signal handlers
@@ -85,7 +86,7 @@ class Foreman::Engine
     @selfpipe[:writer].write_nonblock( '.' )
   rescue Errno::EAGAIN
     # Ignore writes that would block
-  rescue Errno::EINT
+  rescue Errno::EINTR
     # Retry if another signal arrived while writing
     retry
   end
@@ -111,22 +112,22 @@ class Foreman::Engine
   # Handle a TERM signal
   #
   def handle_term_signal
-    puts "SIGTERM received"
-    terminate_gracefully
+    system "SIGTERM received, starting shutdown"
+    @shutdown = true
   end
 
   # Handle an INT signal
   #
   def handle_interrupt
-    puts "SIGINT received"
-    terminate_gracefully
+    system "SIGINT received, starting shutdown"
+    @shutdown = true
   end
 
   # Handle a HUP signal
   #
   def handle_hangup
-    puts "SIGHUP received"
-    terminate_gracefully
+    system "SIGHUP received, starting shutdown"
+    @shutdown = true
   end
 
   # Register a process to be run by this +Engine+
@@ -371,7 +372,7 @@ private
 
   def read_self_pipe
     @selfpipe[:reader].read_nonblock(11)
-  rescue Errno::EAGAIN, Errno::EINTR, Errno::EBADF
+  rescue Errno::EAGAIN, Errno::EINTR, Errno::EBADF, Errno::EWOULDBLOCK
     # ignore
   end
 
@@ -410,19 +411,52 @@ private
     end
   end
 
-  def watch_for_termination
-    pid, status = Process.wait2
+  def wait_for_shutdown_or_child_termination
+    loop do
+      # Stop if it is time to shut down (asked via a signal)
+      break if @shutdown
+
+      # Stop if any of the children died
+      break if check_for_termination
+
+      # Sleep for a moment and do not blow up if any signals are coming our way
+      begin
+        sleep(1)
+      rescue Exception
+        # noop
+      end
+    end
+
+    # Ok, we have exited from the main loop, time to shut down gracefully
+    terminate_gracefully
+  end
+
+  def check_for_termination
+    # Check if any of the children have died off
+    pid, status = begin
+      Process.wait2(-1, Process::WNOHANG)
+    rescue Errno::ECHILD
+      return nil
+    end
+
+    # record the exit status
+    @exitstatus ||= status.exitstatus if status
+
+    # If no childred have died, nothing to do here
+    return nil unless pid
+
+    # Log the information about the process that exited
     output_with_mutex name_for(pid), termination_message_for(status)
+
+    # Delete it from the list of running processes and return its pid
     @running.delete(pid)
-    yield if block_given?
-    pid
-  rescue Errno::ECHILD
+    return pid
   end
 
   def terminate_gracefully
-    return if @terminating
     restore_default_signal_handlers
-    @terminating = true
+
+    # Tell all children to stop gracefully
     if Foreman.windows?
       system  "sending SIGKILL to all processes"
       kill_children "SIGKILL"
@@ -430,12 +464,23 @@ private
       system  "sending SIGTERM to all processes"
       kill_children "SIGTERM"
     end
-    Timeout.timeout(options[:timeout]) do
-      watch_for_termination while @running.length > 0
+
+    # Wait for all children to stop or until the time comes to kill them all
+    start_time = Time.now
+    while Time.now - start_time <= options[:timeout]
+      return if @running.empty?
+      check_for_termination
+
+      # Sleep for a moment and do not blow up if more signals are coming our way
+      begin
+        sleep(0.1)
+      rescue Exception
+        # noop
+      end
     end
-  rescue Timeout::Error
+
+    # Ok, we have no other option than to kill all of our children
     system  "sending SIGKILL to all processes"
     kill_children "SIGKILL"
   end
-
 end
