@@ -51,6 +51,11 @@ describe "Foreman::CLI", :fakefs do
           expect(output).to match(/ps.1   \| PS env var is ps.1/)
         end
       end
+
+      it "fails if process fails" do
+        output = `bundle exec foreman start -f #{resource_path "Procfile.bad"} && echo success`
+        expect(output).not_to include 'success'
+      end
     end
   end
 
@@ -66,7 +71,6 @@ describe "Foreman::CLI", :fakefs do
     end
 
     it "without a Procfile displays an error" do
-      FileUtils.rm_f "Procfile"
       expect(foreman("check")).to eq("ERROR: Procfile does not exist.\n")
     end
   end
